/*
	Client protocol by Catid@pacbell.net
*/

#ifndef CLIENTPROT_H
#define CLIENTPROT_H

#include "datatypes.h"
#include "host.h"

// -1 is used in many places to designate
// the absence of an optional object stub
#ifndef PROT_ERROR
#define PROT_ERROR 0xFFFF
#endif

/*	c2s Message types
	--	Original SubSpace protocol
	00	Start of a special header
	01	Arena login
	02	Leave arena
	03	Position packet
	04	
	05	Death message
	06	Chat message
	07	
	08	Spectate player
	09	Password packet
	0A	
	0B	SSUpdate.EXE request
	0C	map.lvl request
	0D	news.txt request
	0E	(server crash)
	0F	Frequency change
	10	Attach request
	11	
	12	
	13	Flag request
	14	
	15	Drop flags
	16	File transfer
	17	Registration information response
	18	Set ship type
	19	Set personal banner
	1A	Security checksum response
	1B	Security violation
	1C	Drop brick
	1D	(server crash)
	1E	Personal KoTH timer ended
	1F	Fire a ball
	20	Ball request
	21	
	22	Task switch

	--	Additional Continuum protocol
	23	
	24	Continuum password packet
*/

/*	s2c Message types
	--	Original SubSpace protocol
	01	PlayerID change
	02	You are now in the game
	03	Player entering
	04	Player leaving
	05	Player fired a weapon
	06	Player died
	07	Chat
	08	Player took a prize
	09	Player score changed
	0A	Password packet response
	0B	Soccer goal
	0C	
	0D	Set player frequency
	0E	Create turret link
	0F	Arena settings
	10	File transfer
	11	
	12	Flag position
	13	Flag claim
	14	Flag victory
	15	Destroy turret link
	16	Drop flag
	17	
	18	Security checksum
	19	Request file
	1A	Reset score(s)
	1B	Personal ship reset
	1C	Put player in spectator mode
	1D	Player team and ship changed
	1E	Banner flag
	1F	Player banner changed
	20	Collected prize
	21	Brick dropped
	22	Turf flag update
	23	Flag reward granted
	24	
	25	Toggle UFO ship
	26	
	27	Keep-alive
	28	Player position update
	29	Map information
	2A	Compressed map file
	2B	? Happens right after KoTH reset: 2B 10 27 00 00 (10 27 == 10000 points?) (00 00 == player id?)
	2C	KoTH game reset
	2D	
	2E	Power-ball position update
	2F	Arena directory listing
	30	Got zone banner advertisements
	31	You are now past the login sequence

	--	Additional Continuum protocol
	32	Change personal ship coordinates
	33	Custom login failure message
*/


// Wrapper for raw ship type numbers
// c2s (Change ship) 18 __

enum Ship_Types
{
	SHIP_Warbird,		// 0
	SHIP_Javelin,
	SHIP_Spider,
	SHIP_Leviathan,
	SHIP_Terrier,
	SHIP_Weasel,
	SHIP_Lancaster,
	SHIP_Shark,
	SHIP_Spectator		// 8
};


// Password response constants
// s2c (Password response) 0A __ 86 00 00 00 00 00 00 00 E8 9C 42 F1 00 00 00 00 00 00 48 C9 1C 28 7C 37 92 E6 00 00 00 00 20 00 00 00

enum Login_Params
{
	LOG_Continue,			// 0   - Move along.
	LOG_NewUser,			// 1   - Unknown player, continue as new user?
	LOG_InvalidPassword,	// 2   - Invalid password for specified user.  The name you have chosen is probably in use by another player, try picking a different name.
	LOG_FullArena,			// 3   - This arena is currently full, try again later.
	LOG_LockedOut,			// 4   - You have been locked out of SubSpace, for more information inquire on Web BBS.
	LOG_NoPermission,		// 5   - You do not have permission to play in this arena, see Web Site for more information.
	LOG_SpectateOnly,		// 6   - You only have permission to spectate in this arena.
	LOG_TooManyPoints,		// 7   - You have too many points to play in this arena, please choose another arena.
	LOG_SlowConnection,		// 8   - Your connection appears to be too slow to play in this arena.
	LOG_NoPermission2,		// 9   - You do not have permission to play in this arena, see Web Site for more information.
	LOG_NoNewConnections,	// 10  - The server is currently not accepting new connections.
	LOG_InvalidName,		// 11  - Invalid user name entered, please pick a different name.
	LOG_ObsceneName,		// 12  - Possibly offensive user name entered, please pick a different name.
	LOG_BillerDown,			// 13  - NOTICE: Server difficulties; this zone is currently not keeping track of scores.  Your original score will be available later.  However, you are free to play in the zone until we resolve this problem.
	LOG_BusyProcessing,		// 14  - The server is currently busy processing other login requests, please try again in a few moments.
	LOG_ExperiencedOnly,	// 15  - This zone is restricted to experienced players only (ie. certain number of game-hours logged).
	LOG_UsingDemoVersion,	// 16  - You are currently using the demo version.  Your name and score will not be kept track of.
	LOG_TooManyDemos,		// 17  - This arena is currently has(sic) the maximum Demo players allowed, try again later.
	LOG_ClosedToDemos,		// 18  - This arena is closed to Demo players.
	LOG_UnknownResponse		// ... - Unknown response type, please go to Web site for more information and to obtain latest version of the program.
};


// Chat parameters
// c2s (Chat message) 06 __ 00 00 00 MM EE SS SS AA GG EE 00

enum Chat_Modes
{
    MSG_Arena,			// Same for *arena, *zone, **
    MSG_PublicMacro,	// Macro calcs are done client-side
    MSG_Public,			// Public chat
    MSG_Team,			// (//) or (')
    MSG_TeamPrivate,	// Enemy -> Opposing team  (")
    MSG_Private,		// Only shows messages addressed to the bot
    MSG_PlayerWarning,	// Red message, with a name tag
    MSG_RemotePrivate,	// Do not trust the sender with any private information.
    MSG_ServerError,	// Red server errors, without a name tag
    MSG_Channel,		// Arrives in the same format SubSpace displays
	MSG_Unknown			// Same as public chat
};


// Item info concatenated to the position packets
// c2s (Position packet) 03 00 1D 23 02 00 05 00 27 10 FF SI 25 01 01 00 2D 00 3F 01 WW II 3F 01 00 00 00 00 __ __ __ __

union itemInfo
{
	struct
	{
		unsigned shields	: 1;	// Has it or not
		unsigned super		: 1;
		unsigned burst		: 4;	// Item counts
		unsigned repel		: 4;
		unsigned thor		: 4;
		unsigned brick		: 4;
		unsigned decoy		: 4;
		unsigned rocket		: 4;
		unsigned portal		: 4;
		unsigned pack		: 2;	// Probably used somehow
	} s;

	Uint32 n;
};


// Weapon info concatenated to the weapons packets
// c2s (Position packet) 03 00 1D 23 02 00 05 00 27 10 FF SI 25 01 01 00 2D 00 3F 01 __ __ 3F 01 00 00 00 00 II TT EE MM

union weaponInfo
{
	struct
	{
		unsigned type		: 5;	// enum Projectile_Types
		unsigned level		: 2;	// Only for bombs/bullets
		unsigned bouncing	: 1;	// Only for bombs
		unsigned emp		: 1;	// Only for bombs
		unsigned isBomb		: 1;	// ? This is a bomb
		unsigned shrapCount	: 5;	// 0-31
		unsigned fireType	: 1;	// Bombs -> Mines, Bullets -> Multifire
	} s;

	Uint16 n;
};


// Weapon types for weaponInfo.type

enum Projectile_Types
{
	// Seen "in the wild"
	PROJ_None,
	PROJ_Bullet,
	PROJ_BBullet,
	PROJ_Bomb,
	PROJ_PBomb,
	PROJ_Repel,
	PROJ_Decoy,
	PROJ_Burst,
	PROJ_Thor,

	// Internal to the bot
	PROJ_InactiveBullet,
	PROJ_Shrapnel
};


// State info concatenated to the position packets
// c2s (Position packet) 03 00 1D 23 02 00 05 00 27 10 FF __ 25 01 01 00 2D 00 3F 01 WW II 3F 01 00 00 00 00 II TT EE MM

union stateInfo
{
	struct
	{
		unsigned stealth		: 1;
		unsigned cloaked		: 1;
		unsigned xradar			: 1;
		unsigned awarp			: 1;
		unsigned flash			: 1;	// Uncloaking, portaling, etc.
		unsigned safety			: 1;	// In a safety zone
		unsigned ufo			: 1;	// *ufo - Illegal usage caught in sg9+
		unsigned pack			: 1;	// Probably unused
	} s;

	BYTE n;
};


// Security checksum responses, violation codes
// c2s (Violation notification) 1B __

enum Security_Violations
{
	// NOTE: These may only be sent in response to a security checksum request
	SEC_NormalIntegrity,			// 0x00 - Normal integrity
	SEC_SlowFrameDetected,			// 0x01 - Slow frame detected
	SEC_Energy_CurrentOverTop,		// 0x02 - Current energy higher than top energy
	SEC_Energy_TopOverMax,			// 0x03 - Top energy higher than max energy
	SEC_Energy_MaxWithoutPrize,		// 0x04 - Max energy without getting prizes
	SEC_Recharge_CurrentOverMax,	// 0x05 - Recharge rate higher than max recharge rate
	SEC_Recharge_MaxWithoutPrize,	// 0x06 - Max recharge rate without getting prizes
	SEC_Burst_TooMany,				// 0x07 - Too many burst used
	SEC_Repel_TooMany,				// 0x08 - Too many repel used
	SEC_Decoy_TooMany,				// 0x09 - Too many decoy used
	SEC_Thor_TooMany,				// 0x0A - Too many thor used
	SEC_Brick_TooMany,				// 0x0B - Too many wall blocks used
	SEC_Stealth_WithoutPrize,		// 0x0C - Stealth on but never collected
	SEC_Cloak_WithoutPrize,			// 0x0D - Cloak on but never collected
	SEC_XRadar_WithoutPrize,		// 0x0E - XRadar on but never collected
	SEC_AntiWarp_WithoutPrize,		// 0x0F - AntiWarp on but never collected
	SEC_PBombs_WithoutPrize,		// 0x10 - Proximity bombs but never collected
	SEC_BBullets_WithoutPrize,		// 0x11 - Bouncing bullets but never collected
	SEC_Guns_MaxWithoutPrize,		// 0x12 - Max guns without getting prizes
	SEC_Bombs_MaxWithoutPrize,		// 0x13 - Max bombs without getting prizes
	SEC_Specials_OnTooLong,			// 0x14 - Shields or Super on longer than possible

	// NOTE: This block of codes is the exception.  You may send them at any time.
	SEC_ShipWeaponLimitsTooHigh,	// 0x15 - Saved ship weapon limits too high (burst/repel/etc)
	SEC_ShipWeaponLevelTooHigh,		// 0x16 - Saved ship weapon level too high (guns/bombs)
	SEC_LoginChecksumMismatch,		// 0x17 - Login checksum mismatch (program exited)
	SEC_Unknown0,					// 0x18 - Nothing??
	SEC_ShipChecksumMismatch,		// 0x19 - Saved ship checksum mismatch

	// NOTE: These may only be sent in response to a security checksum request
	SEC_SoftICE,					// 0x1A - Softice Debugger Running
	SEC_DataChecksumMismatch,		// 0x1B - Data checksum mismatch
	SEC_ParameterMismatch,			// 0x1C - Parameter mismatch
	SEC_UnknownViolation,			// .... - Unknown integrity violation
	SEC_HighLatency = 0x3C			// 0x3C - Unknown integrity violation (High latency in Continuum)
};


// RegForm player sex field
enum RegForm_Sex
{
	SEX_Male = 'M',
	SEX_Female = 'F'
};


// Chat message soundbytes
// c2s (Chat) 06 09 __ 01 00 MM EE SS SS AA GG EE 00

enum Chat_SoundBytes
{							// Soundcodes courtesy of MGB
	SND_None,				// 0  = Silence
	SND_BassBeep,			// 1  = BEEP!
	SND_TrebleBeep,			// 2  = BEEP!
	SND_ATT,				// 3  = You're not dealing with AT&T
	SND_Discretion,			// 4  = Due to some violent content, parental discretion is advised
	SND_Hallellula,			// 5  = Hallellula
	SND_Reagan,				// 6  = Ronald Reagan
	SND_Inconceivable,		// 7  = Inconceivable
	SND_Churchill,			// 8  = Winston Churchill
	SND_SnotLicker,			// 9  = Listen to me, you pebble farting snot licker
	SND_Crying,				// 10 = Crying
	SND_Burp,				// 11 = Burp
	SND_Girl,				// 12 = Girl
	SND_Scream,				// 13 = Scream
	SND_Fart,				// 14 = Fart1
	SND_Fart2,				// 15 = Fart2
	SND_Phone,				// 16 = Phone ring
	SND_WorldUnderAttack,	// 17 = The world is under attack at this very moment
	SND_Gibberish,			// 18 = Gibberish
	SND_Ooooo,				// 19 = Ooooo
	SND_Geeee,				// 20 = Geeee
	SND_Ohhhh,				// 21 = Ohhhh
	SND_Ahhhh,				// 22 = Awwww
	SND_ThisGameSucks,		// 23 = This game sucks
	SND_Sheep,				// 24 = Sheep
	SND_CantLogIn,			// 25 = I can't log in!
	SND_MessageAlarm,		// 26 = Beep
	SND_StartMusic = 100,	// 100= Start music playing
	SND_StopMusic,			// 101= Stop music
	SND_PlayOnce,			// 102= Play music for 1 iteration then stop
	SND_VictoryBell,		// 103= Victory bell
	SND_Goal				// 104= Goal!
};


// s2c Message handler prototypes

void handleUnknown			(hostMessage * msg);
void handleInGameFlag		(hostMessage * msg);
void handleChat				(hostMessage * msg);
void handleBannerAds		(hostMessage * msg);
void handleCustomMessage	(hostMessage * msg);
void handleFileTransfer		(hostMessage * msg);
void handleFileRequest		(hostMessage * msg);
void handleLoginNext		(hostMessage * msg);
void handleIdent			(hostMessage * msg);
void handleTurfFlagStatus	(hostMessage * msg);
void handlePlayerPrize		(hostMessage * msg);
void handleSelfPrize		(hostMessage * msg);
void handlePlayerEntering	(hostMessage * msg);
void handleSetTeam			(hostMessage * msg);
void handleSetTeamAndShip	(hostMessage * msg);
void handlePlayerBanner		(hostMessage * msg);
void handlePlayerLeaving	(hostMessage * msg);
void handleCreateTurret		(hostMessage * msg);
void handleDeleteTurret		(hostMessage * msg);
void handleBallPosition		(hostMessage * msg);
void handleArenaList		(hostMessage * msg);
void handleFlagVictory		(hostMessage * msg);
void handleScoreReset		(hostMessage * msg);
void handleFlagDrop			(hostMessage * msg);
void handleFlagClaim		(hostMessage * msg);
void handleFlagReward		(hostMessage * msg);
void handlePlayerPosition	(hostMessage * msg);
void handleKoTHReset		(hostMessage * msg);
void handleShipReset		(hostMessage * msg);
void handleArenaSettings	(hostMessage * msg);
void handleChangePosition	(hostMessage * msg);
void handleToggleUFO		(hostMessage * msg);
void handleBrickDrop		(hostMessage * msg);
void handleMapInfo			(hostMessage * msg);
void handleMapFile			(hostMessage * msg);
void handleKeepAlive		(hostMessage * msg);
void handleSpecPlayer		(hostMessage * msg);
void handleFlagPosition		(hostMessage * msg);
void handleSecurity			(hostMessage * msg);
void handleSoccerGoal		(hostMessage * msg);
void handleScoreUpdate		(hostMessage * msg);
void handleWeaponUpdate		(hostMessage * msg);
void handlePlayerDeath		(hostMessage * msg);
void handlePasswordResponse	(hostMessage * msg);
void handleBannerFlag		(hostMessage * msg);


// c2s Message generator prototypes

clientMessage * generateViolation		(Security_Violations code);
clientMessage * generateChangeShip		(Ship_Types ship);
clientMessage * generateSpectate		(Uint16 player);
clientMessage * generateChangeTeam		(Uint16 team);
clientMessage * generateChangeBanner	(BYTE * buffer);
clientMessage * generateDeath			(Uint16 player, Uint16 bounty);
clientMessage * generateChat			(Chat_Modes type, Chat_SoundBytes soundcode, Uint16 player,
										 char * text);
clientMessage * generateRegForm			(char * name, char * email, char * city, char * state,
										 RegForm_Sex sex, BYTE age, bool playAtHome, bool playAtWork,
										 bool playAtSchool, Uint32 processor, char * regName);
clientMessage * generateKoTHReset		();
clientMessage * generateSecurityChecksum(Uint32 parameterChecksum, Uint32 EXEChecksum, Uint32 levelChecksum,
										 Uint16 ping, Uint16 avgPing, Uint16 lowPing, Uint16 highPing,
										 Uint32 weaponCount, Uint16 S2CSlowCurrent, Uint16 S2CFastCurrent,
										 Uint16 S2CSlowTotal, Uint16 S2CFastTotal, bool slowFrame);
clientMessage * generatePassword		(bool newUser, char * name, char * pass, Uint32 machineID,
										 Uint16 timezoneBias, Uint32 permissionID, Uint16 clientVersion);
clientMessage * generatePowerballRequest(Uint32 timestamp, BYTE ball);
clientMessage * generatePowerballUpdate	(Uint32 timestamp, BYTE ball, Uint16 x, Uint16 y,
										 Uint16 xvelocity, Uint16 yvelocity, Uint16 owner);
clientMessage * generateFlagRequest		(Uint16 flag);
clientMessage * generateFlagDrop		();
clientMessage * generateEXERequest		();
clientMessage * generateLevelRequest	();
clientMessage * generateNewsRequest		();
clientMessage * generateArenaLogin		(char * arena, Ship_Types ship, Uint16 xres, Uint16 yres);
clientMessage * generateArenaLeave		();
clientMessage * generateAttachRequest	(Uint16 player);
clientMessage * generateBrickDrop		(Uint16 xtile, Uint16 ytile);
clientMessage * generatePosition		(Uint32 timestamp, BYTE direction, Uint16 x, Uint16 y, Uint16 xvelocity, Uint16 yvelocity,
										 bool stealth, bool cloaked, bool xradar, bool awarp, bool flash, bool safety, bool ufo,
										 Uint16 bounty, Uint16 energy, Projectile_Types type, BYTE level, bool bouncing, bool emp,
										 bool bomb, BYTE shrap, bool secondary, Uint16 timer, Uint16 S2CLag, bool shields, bool super,
										 BYTE burst, BYTE repel, BYTE thor, BYTE brick, BYTE decoy, BYTE rocket, BYTE portal);
clientMessage * generatePosition		(Uint32 timestamp, BYTE direction, Uint16 x, Uint16 y, Uint16 xvelocity, Uint16 yvelocity,
										 bool stealth, bool cloaked, bool xradar, bool awarp, bool flash, bool safety, bool ufo,
										 Uint16 bounty, Uint16 energy, Projectile_Types type, BYTE level, bool bouncing, bool emp,
										 bool bomb, BYTE shrap, bool secondary);
clientMessage * generateFileTransfer	(char * fileName, char * buffer, Uint32 len);

#endif	// CLIENTPROT_H
